//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
// $Id: G4MuPairProduction52.hh,v 1.4 2006/06/29 19:49:24 gunter Exp $
// GEANT4 tag $Name: geant4-08-01-patch-01 $
//
//--------------- G4MuPairProduction52 physics process ------------------
//                by Laszlo Urban, May 1998      
//------------------------------------------------------------------------------
// 10-02-00 modifications , new e.m. structure, L.Urban
// 10-08-01 new methods Store/Retrieve PhysicsTable (mma)
// 29-10-01 all static functions no more inlined (mma) 
// 16-01-03 Migrade to cut per region (V.Ivanchenko)
// 08-08-03 This class is frozen at the release 5.2 (V.Ivanchenko)
// 08-11-04 Remove interface of Store/Retrieve tables (V.Ivantchenko)
// 10-02-06 ComputeMicroscopicCrossSection()  made public (mma)
//------------------------------------------------------------------------------

#ifndef G4MuPairProduction52_h
#define G4MuPairProduction52_h 1

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "G4ios.hh"
#include "globals.hh"
#include "Randomize.hh"
#include "G4VMuEnergyLoss.hh"
#include "G4Track.hh"
#include "G4Step.hh"
#include "G4OrderedTable.hh"
#include "G4PhysicsTable.hh"
#include "G4PhysicsLogVector.hh"
#include "G4MaterialCutsCouple.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

class G4MuPairProduction52 : public G4VMuEnergyLoss

{
  public:

     G4MuPairProduction52(const G4String& processName = "MuPairProd");

    ~G4MuPairProduction52();

     G4bool IsApplicable(const G4ParticleDefinition&);

     void BuildPhysicsTable(const G4ParticleDefinition& ParticleType);

     void BuildLossTable(const G4ParticleDefinition& ParticleType);

     void BuildLambdaTable(const G4ParticleDefinition& ParticleType);

     void PrintInfoDefinition();

     G4double GetMeanFreePath( const G4Track& track,
                               G4double previousStepSize,
                               G4ForceCondition* condition );

     G4VParticleChange *PostStepDoIt(const G4Track& track,
                                     const G4Step& Step  );

     G4double GetDMicroscopicCrossSection(
                                      const G4ParticleDefinition* ParticleType,
                                      G4double KineticEnergy,
                                      G4double AtomicNumber,
                                      G4double PairEnergy);

  public:

     G4double ComputeMeanFreePath(const G4ParticleDefinition* ParticleType,
                                        G4double KineticEnergy,
                                  const G4MaterialCutsCouple* couple);

     void ComputePartialSumSigma(const G4ParticleDefinition* ParticleType,
                                       G4double KineticEnergy,
                                 const G4MaterialCutsCouple* couple);

     virtual G4double ComputeMicroscopicCrossSection(
                                      const G4ParticleDefinition* ParticleType,
                                      G4double KineticEnergy,
                                      G4double AtomicNumber,
                                      G4double ElectronEnergyCut,
                                      G4double PositronEnergyCut);

     G4double ComputeDDMicroscopicCrossSection(
                                      const G4ParticleDefinition* ParticleType,
                                      G4double KineticEnergy,
                                      G4double AtomicNumber,
                                      G4double PairEnergy,
                                      G4double asymmetry);

     G4double ComputeDMicroscopicCrossSection(
                                      const G4ParticleDefinition* ParticleType,
                                      G4double KineticEnergy,
                                      G4double AtomicNumber,
                                      G4double PairEnergy);

  private:

     G4MuPairProduction52 & operator=(const G4MuPairProduction52 &right);
     G4MuPairProduction52(const G4MuPairProduction52&);

     G4double ComputePairLoss( const G4ParticleDefinition* ParticleType,
                               G4double Z,G4double T,G4double ElectronCut,
                                     G4double PositronCut);

     G4Element* SelectRandomAtom(const G4MaterialCutsCouple* couple) const;

     void MakeSamplingTables( const G4ParticleDefinition* ParticleType );

  protected:

     virtual G4double SecondaryEnergyThreshold(size_t index);

  private:

     G4PhysicsTable* theMeanFreePathTable;

     G4OrderedTable PartialSumSigma;

     static G4double LowerBoundLambda; // bining for lambda table
     static G4double UpperBoundLambda;
     static G4int    NbinLambda;
     G4double LowestKineticEnergy,HighestKineticEnergy;
     G4int    TotBin;

     const std::vector<G4double>* electronEnergyCuts;
     const std::vector<G4double>* positronEnergyCuts;

     // tables for sampling
     static G4int nzdat,ntdat,NBIN;
     static G4double zdat[5],tdat[8];
     static G4double ya[1001],proba[5][8][1001];
     static G4double MinPairEnergy;

  public:

    static void SetLowerBoundLambda(G4double val);
    static void SetUpperBoundLambda(G4double val);
    static void SetNbinLambda(G4int n);
    static G4double GetLowerBoundLambda();
    static G4double GetUpperBoundLambda();
    static G4int GetNbinLambda();

};

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "G4MuPairProduction52.icc"

#endif
